@@ -91,7 +91,7 @@ namespace bt
 		{
 			delete fptr;
 			fptr = 0;
-			throw Error(i18n("Cannot open %1 : %2",path,strerror(errno)));
+			throw Error(i18n("Cannot open %1: %2",path,strerror(errno)));
 		}
 		
 		file_size = fptr->size();
@@ -112,12 +112,14 @@ namespace bt
 		if (!fptr)
 		{
 		//	Out(SYS_DIO|LOG_DEBUG) << "Reopening " << path << endl;
+			if (!OpenFileAllowed()) 
+				return 0; // Running out of file descriptors, force buffered mode
 			openFile(mode);
 		}
 		
 		if (read_only && mode != READ)
 		{
-			throw Error(i18n("Cannot open %1 for writing : readonly filesystem",path));
+			throw Error(i18n("Cannot open %1 for writing: readonly filesystem",path));
 		}
 		
 		if (off + size > max_size)
@@ -245,18 +247,18 @@ namespace bt
 		}
 		
 		if (read_only)
-			throw Error(i18n("Cannot open %1 for writing : readonly filesystem",path));
+			throw Error(i18n("Cannot open %1 for writing: readonly filesystem",path));
 		
 		
 		if (file_size + to_write > max_size)
 		{
 			Out(SYS_DIO|LOG_DEBUG) << "Warning : writing past the end of " << path << endl;
 			Out(SYS_DIO|LOG_DEBUG) << (file_size + to_write) << " " << max_size << endl;
-			throw Error(i18n("Cannot expand file %1 : attempting to grow the file beyond the maximum size",path));
+			throw Error(i18n("Cannot expand file %1: attempting to grow the file beyond the maximum size",path));
 		}
 		
 		if (!fptr->resize(file_size + to_write))
-			throw Error(i18n("Cannot expand file %1 : %2",path,fptr->errorString()));
+			throw Error(i18n("Cannot expand file %1: %2",path,fptr->errorString()));
 		
 		file_size = fptr->size();
 	}
@@ -286,18 +288,17 @@ namespace bt
 		{
 			CacheFile::Entry & e = mappings[ptr];
 #ifdef HAVE_MUNMAP64
-			ret = munmap64((char*)e.ptr,e.size);
+			ret = munmap64(e.ptr,e.size);
 #else
-			ret = munmap((char*)e.ptr,e.size);
+			ret = munmap(e.ptr,e.size);
 #endif
+			if (ret < 0)
+				Out(SYS_DIO|LOG_IMPORTANT) << QString("Munmap failed with error %1 : %2").arg(errno).arg(strerror(errno)) << endl;
 			
 			mappings.remove(ptr);
 			// no mappings, close temporary
 			if (mappings.count() == 0)
 				closeTemporary();
-			
-			if (ret < 0)
-				Out(SYS_DIO|LOG_IMPORTANT) << QString("Munmap failed with error %1 : %2").arg(errno).arg(strerror(errno)) << endl;
 		}
 #endif
 	}
@@ -329,15 +330,9 @@ namespace bt
 			fptr->unmap((uchar*)e.ptr);
 #else
 #ifdef HAVE_MUNMAP64
-			if (e.diff > 0)
-				ret = munmap64((char*)e.ptr - e.diff,e.size);
-			else
-				ret = munmap64(e.ptr,e.size);
+			ret = munmap64(e.ptr,e.size);
 #else
-			if (e.diff > 0)
-				ret = munmap((char*)e.ptr - e.diff,e.size);
-			else
-				ret = munmap(e.ptr,e.size);
+			ret = munmap(e.ptr,e.size);
 #endif // HAVE_MUNMAP64
 #endif // Q_OS_WIN
 			e.thing->unmapped();
@@ -348,7 +343,7 @@ namespace bt
 			if (ret < 0)
 			{
 				Out(SYS_DIO|LOG_IMPORTANT) << QString("Munmap failed with error %1 : %2").arg(errno).arg(strerror(errno)) << endl;
-			}	
+			}
 		}	
 	}
 		
@@ -382,12 +377,12 @@ namespace bt
 		
 		if (off >= file_size || off >= max_size)
 		{
-			throw Error(i18n("Error : Reading past the end of the file %1",path));
+			throw Error(i18n("Error: Reading past the end of the file %1",path));
 		}
 		
 		// jump to right position
 		if (!fptr->seek(off))
-			throw Error(i18n("Failed to seek file %1 : %2",path,fptr->errorString()));
+			throw Error(i18n("Failed to seek file %1: %2",path,fptr->errorString()));
 		
 		Uint32 sz = 0;
 		if ((sz = fptr->read((char*)buf,size)) != size)
@@ -416,7 +411,7 @@ namespace bt
 		}
 		
 		if (read_only)
-			throw Error(i18n("Cannot open %1 for writing : readonly filesystem",path));
+			throw Error(i18n("Cannot open %1 for writing: readonly filesystem",path));
 		
 		if (off + size > max_size)
 		{
@@ -434,11 +429,11 @@ namespace bt
 		
 		// jump to right position
 		if (!fptr->seek(off))
-			throw Error(i18n("Failed to seek file %1 : %2",path,fptr->errorString()));
+			throw Error(i18n("Failed to seek file %1: %2",path,fptr->errorString()));
 		
 		if (fptr->write((const char*)buf,size) != size)
 		{
-			throw Error(i18n("Failed to write to file %1 : %2",path,fptr->errorString()));
+			throw Error(i18n("Failed to write to file %1: %2",path,fptr->errorString()));
 		}
 		
 		if (close_again)
@@ -484,7 +479,7 @@ namespace bt
 			if (close_again)
 				closeTemporary();
 			
-			throw Error(i18n("Cannot open %1 for writing : readonly filesystem",path));
+			throw Error(i18n("Cannot open %1 for writing: readonly filesystem",path));
 		}
 
 		try
@@ -505,7 +500,7 @@ namespace bt
 		}
 		catch (bt::Error & e)
 		{
-			throw Error(i18n("Cannot preallocate diskspace : %1",strerror(errno)));
+			throw Error(i18n("Cannot preallocate diskspace: %1",strerror(errno)));
 		}
 
 		file_size = FileSize(fd);
