@@ -19,6 +19,8 @@
  ***************************************************************************/
 #include "movedatafilesjob.h"
 #include <QFileInfo>
+#include <KLocale>
+#include <kjobtrackerinterface.h>
 #include <kio/jobuidelegate.h>
 #include <util/log.h>
 #include <util/fileops.h>
@@ -27,13 +29,30 @@
 
 namespace bt
 {
+	static ResourceManager move_data_files_slot(1);
 
-	MoveDataFilesJob::MoveDataFilesJob() : Job(true,0),err(false),active_job(0),running_recovery_jobs(0)
-	{}
+	MoveDataFilesJob::MoveDataFilesJob() 
+		: Job(true,0),
+		Resource(&move_data_files_slot,"MoveDataFilesJob"),
+		err(false),
+		active_job(0),
+		running_recovery_jobs(0),
+		bytes_moved(0),
+		total_bytes(0),
+		bytes_moved_current_file(0)
+	{
+	}
 
 	
 	MoveDataFilesJob::MoveDataFilesJob(const QMap< TorrentFileInterface*, QString >& fmap) 
-		: Job(true,0),err(false),active_job(0),running_recovery_jobs(0)
+		: Job(true,0),
+		Resource(&move_data_files_slot,"MoveDataFilesJob"),
+		err(false),
+		active_job(0),
+		running_recovery_jobs(0),
+		bytes_moved(0),
+		total_bytes(0),
+		bytes_moved_current_file(0)
 	{
 		file_map = fmap;
 		QMap<TorrentFileInterface*,QString>::const_iterator i = file_map.constBegin();
@@ -86,6 +105,8 @@ namespace bt
 		}
 		else
 		{
+			bytes_moved += bytes_moved_current_file;
+			bytes_moved_current_file = 0;
 			success.insert(active_src,active_dst);
 			active_src = active_dst = QString();
 			active_job = 0;
@@ -104,9 +125,29 @@ namespace bt
 	
 	void MoveDataFilesJob::start()
 	{
-		startMoving();
+		registerWithTracker();
+		QMap<QString,QString>::iterator i = todo.begin();
+		while (i != todo.end())
+		{
+			QFileInfo fi(i.key());
+			total_bytes += fi.size();
+			i++;
+		}
+		setTotalAmount(KJob::Bytes,total_bytes);
+		move_data_files_slot.add(this);
+		if (!active_job)
+		{
+			description(this, i18n("Waiting for other move jobs to finish"),
+				qMakePair(i18nc("The source of a file operation", "Source"), active_src),
+				qMakePair(i18nc("The destination of a file operation", "Destination"), active_dst));
+			emitSpeed(0);
+		}
 	}
 	
+	void MoveDataFilesJob::acquired()
+	{
+		startMoving();
+	}
 	
 	void MoveDataFilesJob::kill(bool quietly)
 	{
@@ -123,14 +164,22 @@ namespace bt
 			return;
 		}
 			
-		QMap<QString,QString>::iterator i = todo.begin();	
-		active_job = KIO::file_move(KUrl(i.key()),KUrl(i.value()),-1/*,KIO::HideProgressInfo*/);
+		QMap<QString,QString>::iterator i = todo.begin();
+		active_job = KIO::file_move(KUrl(i.key()),KUrl(i.value()),-1,KIO::HideProgressInfo);
 		active_src = i.key();
 		active_dst = i.value();
 		Out(SYS_GEN|LOG_DEBUG) << "Moving " << active_src << " -> " << active_dst << endl;
 		connect(active_job,SIGNAL(result(KJob*)),this,SLOT(onJobDone(KJob*)));
 		connect(active_job,SIGNAL(canceled(KJob*)),this,SLOT(onCanceled(KJob*)));
+		connect(active_job,SIGNAL(processedAmount(KJob*,KJob::Unit,qulonglong)),
+				this,SLOT(onTransferred(KJob*,KJob::Unit,qulonglong)));
+		connect(active_job,SIGNAL(speed(KJob*,ulong)),
+				this,SLOT(onSpeed(KJob*,ulong)));
 		todo.erase(i);
+		
+		description(this, i18nc("@title job","Moving"),
+					qMakePair(i18nc("The source of a file operation", "Source"), active_src),
+					qMakePair(i18nc("The destination of a file operation", "Destination"), active_dst));
 		addSubjob(active_job);
 	}
 	
@@ -164,5 +213,23 @@ namespace bt
 		if (running_recovery_jobs <= 0)
 			emitResult();
 	}
+	
+	void MoveDataFilesJob::onTransferred(KJob *job, KJob::Unit unit, qulonglong amount)
+	{
+		Q_UNUSED(job);
+		if (unit == KJob::Bytes)
+		{
+			bytes_moved_current_file = amount;
+			setProcessedAmount(unit, bytes_moved + bytes_moved_current_file);
+		}
+	}
+	
+	void MoveDataFilesJob::onSpeed(KJob* job, unsigned long speed)
+	{
+		Q_UNUSED(job);
+		emitSpeed(speed);
+	}
+
+
 }
 #include "movedatafilesjob.moc"
