@@ -59,7 +59,8 @@ using namespace bt;
 namespace net
 {
 
-	Socket::Socket(int fd,int ip_version) : m_fd(fd),m_ip_version(ip_version),r_poll_index(-1),w_poll_index(-1)
+	Socket::Socket(int fd,int ip_version) 
+		: SocketDevice(bt::TCP), m_fd(fd),m_ip_version(ip_version),r_poll_index(-1),w_poll_index(-1)
 	{
 		// check if the IP version is 4 or 6
 		if (m_ip_version != 4 && m_ip_version != 6)
@@ -75,7 +76,8 @@ namespace net
 		cacheAddress();
 	}
 	
-	Socket::Socket(bool tcp,int ip_version) : m_fd(-1),m_ip_version(ip_version),r_poll_index(-1),w_poll_index(-1)
+	Socket::Socket(bool tcp,int ip_version) 
+		: SocketDevice(bt::TCP),m_fd(-1),m_ip_version(ip_version),r_poll_index(-1),w_poll_index(-1)
 	{
 		// check if the IP version is 4 or 6
 		if (m_ip_version != 4 && m_ip_version != 6)
@@ -157,7 +159,10 @@ namespace net
 		
 	bool Socket::connectTo(const Address & a)
 	{
-		if (::connect(m_fd,a.address(),a.length()) < 0)
+		int len = 0;
+		struct sockaddr_storage ss;
+		a.toSocketAddress(&ss, len);
+		if (::connect(m_fd,(struct sockaddr*)&ss,len) < 0)
 		{
 #ifndef Q_WS_WIN
 			if (errno == EINPROGRESS)
@@ -183,7 +188,12 @@ namespace net
 	
 	bool Socket::bind(const QString & ip,Uint16 port,bool also_listen)
 	{
-		int val = 1, no = 0;
+		return bind(net::Address(ip, port), also_listen);
+	}
+	
+	bool Socket::bind(const net::Address& addr, bool also_listen)
+	{
+		int val = 1;
 #ifndef Q_WS_WIN
 		if (setsockopt(m_fd,SOL_SOCKET,SO_REUSEADDR,&val,sizeof(int)) < 0)
 #else
@@ -192,53 +202,25 @@ namespace net
 		{
 			Out(SYS_CON|LOG_NOTICE) << QString("Failed to set the reuseaddr option : %1").arg(strerror(errno)) << endl;
 		}
-		// Bind this socket to IPv4 as well. This is a workaround for systems
-		// which support IPv4-mapped addresses but have them disabled by
-		// default (e.g. net.ipv6.bindv6only=1 on Debian Linux).
-		setsockopt(m_fd,SOL_IPV6,IPV6_V6ONLY,&no,sizeof(no));
 
-		net::Address addr(ip,port);
-		if (::bind(m_fd,addr.address(),addr.length()) != 0)
+		int len = 0;
+		struct sockaddr_storage ss;
+		addr.toSocketAddress(&ss, len);
+		if (::bind(m_fd, (struct sockaddr*)&ss, len) != 0)
 		{
-			Out(SYS_CON|LOG_IMPORTANT) << QString("Cannot bind to port %1:%2 : %3").arg(ip).arg(port).arg(strerror(errno)) << endl;
+			Out(SYS_CON|LOG_IMPORTANT) << QString("Cannot bind to port %1:%2 : %3")
+				.arg(addr.toString())
+				.arg(addr.port())
+				.arg(strerror(errno)) << endl;
 			return false;
 		}
 
 		if (also_listen && listen(m_fd,SOMAXCONN) < 0)
 		{
-			Out(SYS_CON|LOG_IMPORTANT) << QString("Cannot listen to port %1:%2 : %3").arg(ip).arg(port).arg(strerror(errno)) << endl;
-			return false;
-		}
-
-		m_state = BOUND;
-		return true;
-	}
-	
-	bool Socket::bind(const net::Address& addr, bool also_listen)
-	{
-		int val = 1, no = 0;
-#ifndef Q_WS_WIN
-		if (setsockopt(m_fd,SOL_SOCKET,SO_REUSEADDR,&val,sizeof(int)) < 0)
-#else
-		if (setsockopt(m_fd,SOL_SOCKET,SO_REUSEADDR,(char *)&val,sizeof(int)) < 0)
-#endif 
-		{
-			Out(SYS_CON|LOG_NOTICE) << QString("Failed to set the reuseaddr option : %1").arg(strerror(errno)) << endl;
-		}
-		// Bind this socket to IPv4 as well. This is a workaround for systems
-		// which support IPv4-mapped addresses but have them disabled by
-		// default (e.g. net.ipv6.bindv6only=1 on Debian Linux).
-		setsockopt(m_fd,SOL_IPV6,IPV6_V6ONLY,&no,sizeof(no));
-		
-		if (::bind(m_fd,addr.address(),addr.length()) != 0)
-		{
-			Out(SYS_CON|LOG_IMPORTANT) << QString("Cannot bind to port %1:%2 : %3").arg(addr.ipAddress().toString()).arg(addr.port()).arg(strerror(errno)) << endl;
-			return false;
-		}
-		
-		if (also_listen && listen(m_fd,5) < 0)
-		{
-			Out(SYS_CON|LOG_IMPORTANT) << QString("Cannot listen to port %1:%2 : %3").arg(addr.ipAddress().toString()).arg(addr.port()).arg(strerror(errno)) << endl;
+			Out(SYS_CON|LOG_IMPORTANT) << QString("Cannot listen to port %1:%2 : %3")
+				.arg(addr.toString())
+				.arg(addr.port())
+				.arg(strerror(errno)) << endl;
 			return false;
 		}
 		
@@ -294,7 +276,10 @@ namespace net
 	
 	int Socket::sendTo(const bt::Uint8* buf,int len,const Address & a)
 	{
-		int ret = ::sendto(m_fd,(char*)buf,len,0,a.address(),a.length());
+		int alen = 0;
+		struct sockaddr_storage ss;
+		a.toSocketAddress(&ss, alen);
+		int ret = ::sendto(m_fd,(char*)buf,len,0,(struct sockaddr*)&ss,alen);
 		if (ret < 0)
 		{
 			if (errno == EAGAIN || errno == EWOULDBLOCK)
@@ -321,7 +306,7 @@ namespace net
 			Out(SYS_CON|LOG_DEBUG) << "Receive error : " << QString(strerror(errno)) << endl;
 			return 0;
 		}
-		a = KNetwork::KInetSocketAddress((const struct sockaddr*) &ss,slen);
+		a = ss;
 		return ret;
 	}
 	
@@ -336,7 +321,7 @@ namespace net
 			Out(SYS_CON|LOG_DEBUG) << "Accept error : " << QString(strerror(errno)) << endl;
 			return -1;
 		}
-		a = KNetwork::KInetSocketAddress((const struct sockaddr*) &ss,slen);
+		a =  net::Address(&ss);
 		
 		Out(SYS_CON|LOG_DEBUG) << "Accepted connection from " << a.toString() << endl;
 		return sfd;
@@ -368,11 +353,17 @@ namespace net
 		}
 		else
 		{
-#ifdef __GNUC__
-#warning "Find way to set IPv6 traffic class"
+#if defined(IPV6_TCLASS)
+			int c = type_of_service;
+			if (setsockopt(m_fd,IPPROTO_IPV6,IPV6_TCLASS,&c,sizeof(c)) < 0)
+			{
+				Out(SYS_CON|LOG_NOTICE) << QString("Failed to set traffic class to %1 : %2")
+						.arg((int)type_of_service).arg(strerror(errno)) << endl;
+				return false;
+			}
 #endif
-			return true;
 		}
+		
 		return true;
 	}
 	
@@ -419,11 +410,7 @@ namespace net
 		
 		if (getpeername(m_fd,(struct sockaddr*)&ss,&sslen) == 0)
 		{
-			// If it is a IPv6 mapped address convert to IPv4
-			KNetwork::KInetSocketAddress tmp((struct sockaddr*)&ss,sslen);
-			if (tmp.ipVersion() == 6 && tmp.ipAddress().isV4Mapped())
-				tmp.setHost(KNetwork::KIpAddress(tmp.ipAddress().IPv4Addr(true)));
-			addr = tmp;
+			addr = net::Address(&ss);
 		}
 	}
 
@@ -433,15 +420,9 @@ namespace net
 		socklen_t sslen = sizeof(ss);
 		
 		if (getsockname(m_fd,(struct sockaddr*)&ss,&sslen) == 0)
-		{
-			// If it is a IPv6 mapped address convert to IPv4
-			KNetwork::KInetSocketAddress tmp((struct sockaddr*)&ss,sslen);
-			if (tmp.ipVersion() == 6 && tmp.ipAddress().isV4Mapped())
-				tmp.setHost(KNetwork::KIpAddress(tmp.ipAddress().IPv4Addr(true)));
-			return tmp;
-		}
-
-		return Address();
+			return net::Address(&ss);
+		else
+			return Address();
 	}
 
 	int Socket::take()
