@@ -24,7 +24,7 @@
 #include <util/log.h>
 #include <util/functions.h>
 #include <net/address.h>
-#include <mse/streamsocket.h>
+#include <mse/encryptedpacketsocket.h>
 #include <diskio/chunk.h>
 #include <download/piece.h>
 #include <download/request.h>
@@ -34,7 +34,6 @@
 #include <torrent/server.h>
 #include <torrent/torrent.h>
 #include "packetreader.h"
-#include "packetwriter.h"
 #include "peerdownloader.h"
 #include "peeruploader.h"
 #include "utpex.h"
@@ -52,49 +51,37 @@ namespace bt
 	bool Peer::resolve_hostname = true;
 	
 	
-	Peer::Peer(mse::StreamSocket* sock,const PeerID & peer_id,
+	Peer::Peer(mse::EncryptedPacketSocket::Ptr sock,const PeerID & peer_id,
 			   Uint32 num_chunks,Uint32 chunk_size,Uint32 support,bool local,PeerManager* pman)
-	: sock(sock),pieces(num_chunks),peer_id(peer_id),pman(pman)
+	: PeerInterface(peer_id,num_chunks),sock(sock),pman(pman)
 	{
 		id = peer_id_counter;
 		peer_id_counter++;
 		ut_pex_id = 0;
-		preader = new PacketReader(this);
-		stats.choked = true;
-		stats.interested = stats.am_interested = false;
-		killed = false;
+		
+		Uint32 max_packet_len = 9 + MAX_PIECE_LEN; // size of piece packet
+		Uint32 bitfield_length = num_chunks / 8 + 1 + (num_chunks % 8 == 0 ? 0 : 1);
+		if (bitfield_length > max_packet_len) // bitfield can be longer
+			max_packet_len = bitfield_length;
+		
+		// to be future proof use 10 times the max packet length, 
+		// in case some extension message comes along which is larger
+		preader = new PacketReader(this,10*max_packet_len);
 		downloader = new PeerDownloader(this,chunk_size);
 		uploader = new PeerUploader(this);
 		
-		
 		stalled_timer.update();
-		pwriter = new PacketWriter(this);
-		time_choked = CurrentTime();
-		time_unchoked = 0;
+		
 		
 		connect_time = QTime::currentTime();
-		//sock->attachPeer(this);
 		stats.client = peer_id.identifyClient();
 		stats.ip_address = getIPAddresss();
-		stats.choked = true;
-		paused = false;
-		stats.interested = false;
-		stats.am_interested = false;
-		stats.download_rate = 0;
-		stats.upload_rate = 0;
-		stats.perc_of_file = 0;
-		stats.snubbed = false;
 		stats.dht_support = support & DHT_SUPPORT;
 		stats.fast_extensions = support & FAST_EXT_SUPPORT;
 		stats.extension_protocol = support & EXT_PROT_SUPPORT;
-		stats.bytes_downloaded = stats.bytes_uploaded = 0;
-		stats.aca_score = 0.0;
-		stats.has_upload_slot = false;
-		stats.num_up_requests = stats.num_down_requests = 0;
 		stats.encrypted = sock->encrypted();
 		stats.local = local;
-		stats.max_request_queue = 0;
-		bytes_downloaded_since_unchoke = 0;
+		stats.transport_protocol = sock->socketDevice()->transportProtocol();
 		
 		if (stats.ip_address == "0.0.0.0")
 		{
@@ -103,8 +90,9 @@ namespace bt
 		}
 		else
 		{
-			sock->startMonitoring(preader,pwriter);
+			sock->startMonitoring(preader);
 		}
+		
 		pex_allowed = stats.extension_protocol;
 		extensions.setAutoDelete(true);
 		
@@ -119,10 +107,12 @@ namespace bt
 
 	Peer::~Peer()
 	{
+		// Seeing that we are going to delete the preader object,
+		// call stopMonitoring, in some situations it is possible that the socket
+		// is only deleted later because the authenticate object still has a reference to it
+		sock->stopMonitoring();
 		delete uploader;
 		delete downloader;
-		delete sock;
-		delete pwriter;
 		delete preader;
 	}
 	
@@ -157,7 +147,7 @@ namespace bt
 				
 				if (!stats.choked)
 				{
-					time_choked = CurrentTime();
+					stats.time_choked = CurrentTime();
 				}
 				stats.choked = true;
 				downloader->choked();
@@ -172,7 +162,7 @@ namespace bt
 				
 				if (stats.choked)
 				{
-					time_unchoked = CurrentTime();
+					stats.time_unchoked = CurrentTime();
 					bytes_downloaded_since_unchoke = 0;
 				}
 				
@@ -257,7 +247,7 @@ namespace bt
 					if (stats.has_upload_slot)
 						uploader->addRequest(r);
 					else if (stats.fast_extensions)
-						pwriter->sendReject(r);
+						sendReject(r);
 				//	Out(SYS_CON|LOG_DEBUG) << "REQUEST " << r.getIndex() << " " << r.getOffset() << endl;
 				}
 				break;
@@ -298,6 +288,7 @@ namespace bt
 							  ReadUint32(tmp_buf,9),
 							  downloader);
 					uploader->removeRequest(r);
+					sock->doNotSendPiece(r, stats.fast_extensions);
 				}
 				break;
 			case REJECT_REQUEST:
@@ -370,7 +361,7 @@ namespace bt
 		downloader->cancelAll();
 		// choke the peer and tell it we are not interested
 		choke();
-		pwriter->sendNotInterested();
+		sendNotInterested();
 		paused = true;
 	}
 	
@@ -469,6 +460,11 @@ namespace bt
 			{
 				stats.max_request_queue = val->data().toInt();
 			}
+			
+			if ((val = dict->getValue("upload_only")))
+			{
+				stats.partial_seed = val->data().toInt() == 1;
+			}
 		}
 		catch (...)
 		{
@@ -534,10 +530,10 @@ namespace bt
 			return;
 		}
 		
-		sock->updateSpeeds();
+		sock->updateSpeeds(bt::CurrentTime());
 		preader->update();
 		
-		Uint32 data_bytes = pwriter->getUploadedDataBytes();
+		Uint32 data_bytes = sock->dataBytesUploaded();
 		
 		if (data_bytes > 0)
 		{
@@ -559,8 +555,15 @@ namespace bt
 		// if no data is being sent or recieved, and there are pending requests
 		// increment the connection stalled timer
 		if (getUploadRate() > 100 || getDownloadRate() > 100 || 
-		   (uploader->getNumRequests() == 0 && downloader->getNumRequests() == 0) )
+		   (uploader->getNumRequests() == 0 && sock->numPendingPieceUploads() == 0 && downloader->getNumRequests() == 0) )
 			stalled_timer.update();
+
+		stats.download_rate = this->getDownloadRate();
+		stats.upload_rate = this->getUploadRate();
+		stats.perc_of_file = this->percentAvailable();
+		stats.snubbed = this->isSnubbed();
+		stats.num_up_requests = uploader->getNumRequests() + sock->numPendingPieceUploads();
+		stats.num_down_requests = downloader->getNumRequests();
 	}
 	
 	bool Peer::isStalled() const
@@ -574,11 +577,6 @@ namespace bt
 		return snub_timer.getElapsedSinceUpdate() >= 2*60*1000 && stats.num_down_requests > 0;
 	}
 
-	bool Peer::isSeeder() const
-	{
-		return pieces.allOn();
-	}
-
 	QString Peer::getIPAddresss() const
 	{
 		if (sock)
@@ -623,17 +621,6 @@ namespace bt
 		return (float)bytes / (float)tbytes * 100.0;
 	}
 
-	const PeerInterface::Stats & Peer::getStats() const
-	{
-		stats.download_rate = this->getDownloadRate();
-		stats.upload_rate = this->getUploadRate();
-		stats.perc_of_file = this->percentAvailable();
-		stats.snubbed = this->isSnubbed();
-		stats.num_up_requests = uploader->getNumRequests();
-		stats.num_down_requests = downloader->getNumRequests();
-		return stats;
-	}
-	
 	void Peer::setACAScore(double s)
 	{
 		stats.aca_score = s;
@@ -644,7 +631,7 @@ namespace bt
 		if (!stats.has_upload_slot)
 			return;
 		
-		pwriter->sendChoke();
+		sendChoke();
 		uploader->clearAllRequests();
 	}
 	
@@ -677,7 +664,7 @@ namespace bt
 		pex_allowed = on;
 	}
 	
-	void Peer::sendExtProtHandshake(Uint16 port, Uint32 metadata_size)
+	void Peer::sendExtProtHandshake(Uint16 port, Uint32 metadata_size, bool partial_seed)
 	{
 		if (!stats.extension_protocol)
 			return;
@@ -704,15 +691,17 @@ namespace bt
 			enc.write(metadata_size);
 		}
 		
+		enc.write("upload_only", partial_seed ? "1" : "0");
 		enc.write(QString("v")); enc.write(bt::GetVersionString());
 		enc.end();
-		pwriter->sendExtProtMsg(0,arr);
+		sendExtProtMsg(0,arr);
 	}
 
 	
 	void Peer::setGroupIDs(Uint32 up_gid,Uint32 down_gid)
 	{
-		sock->setGroupIDs(up_gid,down_gid);
+		sock->setGroupID(up_gid, true);
+		sock->setGroupID(down_gid, false);
 	}
 		
 	void Peer::resolved(const QString & hinfo)
@@ -750,6 +739,133 @@ namespace bt
 	}
 
 
-}
+	void Peer::sendChoke()
+	{
+		if (!stats.has_upload_slot)
+			return;
+		
+		sock->addPacket(Packet::Ptr(new Packet(CHOKE)));
+		stats.has_upload_slot = false;
+	}
 
-#include "peer.moc"
+	void Peer::sendUnchoke()
+	{
+		if (stats.has_upload_slot)
+			return;
+		
+		sock->addPacket(Packet::Ptr(new Packet(UNCHOKE)));
+		stats.has_upload_slot = true;
+	}
+
+	void Peer::sendEvilUnchoke()
+	{
+		sock->addPacket(Packet::Ptr(new Packet(UNCHOKE)));
+		stats.has_upload_slot = false;
+	}
+
+	void Peer::sendInterested()
+	{
+		if (stats.am_interested == true)
+			return;
+		
+		sock->addPacket(Packet::Ptr(new Packet(INTERESTED)));
+		stats.am_interested = true;
+	}
+
+	void Peer::sendNotInterested()
+	{
+		if (stats.am_interested == false)
+			return;
+		
+		sock->addPacket(Packet::Ptr(new Packet(NOT_INTERESTED)));
+		stats.am_interested = false;
+	}
+
+	void Peer::sendRequest(const Request & r)
+	{
+		sock->addPacket(Packet::Ptr(new Packet(r,bt::REQUEST)));
+	}
+
+	void Peer::sendCancel(const Request & r)
+	{
+		sock->addPacket(Packet::Ptr(new Packet(r,bt::CANCEL)));
+	}
+
+	void Peer::sendReject(const Request & r)
+	{
+		sock->addPacket(Packet::Ptr(new Packet(r,bt::REJECT_REQUEST)));
+	}
+
+	void Peer::sendHave(Uint32 index)
+	{
+		sock->addPacket(Packet::Ptr(new Packet(index,bt::HAVE)));
+	}
+
+	void Peer::sendPort(Uint16 port)
+	{
+		sock->addPacket(Packet::Ptr(new Packet(port)));
+	}
+
+	void Peer::sendBitSet(const BitSet & bs)
+	{
+		sock->addPacket(Packet::Ptr(new Packet(bs)));
+	}
+
+	void Peer::sendHaveAll()
+	{
+		sock->addPacket(Packet::Ptr(new Packet(bt::HAVE_ALL)));
+	}
+
+	void Peer::sendHaveNone()
+	{
+		sock->addPacket(Packet::Ptr(new Packet(bt::HAVE_NONE)));
+	}
+
+	void Peer::sendSuggestPiece(Uint32 index)
+	{
+		sock->addPacket(Packet::Ptr(new Packet(index,bt::SUGGEST_PIECE)));
+	}
+
+	void Peer::sendAllowedFast(Uint32 index)
+	{
+		sock->addPacket(Packet::Ptr(new Packet(index,bt::ALLOWED_FAST)));
+	}
+
+	bool Peer::sendChunk(Uint32 index,Uint32 begin,Uint32 len,Chunk * ch)
+	{
+		//		Out() << "sendChunk " << index << " " << begin << " " << len << endl;
+		if (begin >= ch->getSize() || begin + len > ch->getSize())
+		{
+			Out(SYS_CON|LOG_NOTICE) << "Warning : Illegal piece request" << endl;
+			Out(SYS_CON|LOG_NOTICE) << "\tChunk : index " << index << " size = " << ch->getSize() << endl;
+			Out(SYS_CON|LOG_NOTICE) << "\tPiece : begin = " << begin << " len = " << len << endl;
+			return false;
+		}
+		else if (!ch)
+		{
+			Out(SYS_CON|LOG_NOTICE) << "Warning : attempted to upload an invalid chunk" << endl;
+			return false;
+		}
+		else
+		{
+			/*		Out(SYS_CON|LOG_DEBUG) << QString("Uploading %1 %2 %3 %4 %5")
+			*			.arg(index).arg(begin).arg(len).arg((quint64)ch,0,16).arg((quint64)ch->getData(),0,16) 
+			*			<< endl;;
+			*/
+			sock->addPacket(Packet::Ptr(new Packet(index,begin,len,ch)));
+			return true;
+		}
+	}
+
+	void Peer::sendExtProtMsg(Uint8 id,const QByteArray & data)
+	{
+		sock->addPacket(Packet::Ptr(new Packet(id,data)));
+	}
+	
+	void Peer::clearPendingPieceUploads()
+	{
+		sock->clearPieces(stats.fast_extensions);
+	}
+
+
+}
